@@ -759,7 +759,7 @@ impl CircuitNode for Rearrange {
     ) -> Result<Tensor, super::TensorEvalError> {
         Ok(einops_repeat(
             &tensors[0],
-            self.spec.to_einops_string(),
+            self.spec.to_einops_string(false),
             self.spec.letter_sizes(),
         )?)
     }
@@ -807,6 +807,23 @@ impl Rearrange {
 
         Ok(out.into_init())
     }
+
+    #[staticmethod]
+    pub fn from_einops_string(
+        node: CircuitRc,
+        string: &str,
+        name: Option<String>,
+    ) -> Result<Self, CircuitConstructionError> {
+        Rearrange::try_new(
+            node,
+            RearrangeSpec::from_einops_string(string).ok_or(
+                CircuitConstructionError::RearrangeStringInvalid {
+                    string: string.to_owned(),
+                },
+            )?,
+            name,
+        )
+    }
 }
 
 /// Index indexes a node dimwise.
