@@ -1,23 +1,29 @@
-use crate::{hashmaps::FxHashMap as HashMap, timed};
+use crate::hashmaps::FxHashMap as HashMap;
 
 use super::{
     circuit_utils::{count_nodes, toposort_circuit, total_flops},
     prelude::*,
     repr::ReprWrapper,
-    repr::{repr_op_string, rustfmt_string, RustRepr},
+    repr::{repr_op_string, RustRepr},
     HashBytes,
 };
-use base16::encode_lower;
+
 use num_bigint::BigUint;
 use pyo3::pyfunction;
 
-pub fn repr_circuit_deep_compiler(circuit: &Circuit) -> String {
+pub fn repr_circuit_deep_compiler(
+    circuit: &Circuit,
+    bijection: bool,
+    shape_only_necessesary: bool,
+) -> String {
     let mut seen_hashes: HashMap<HashBytes, String> = HashMap::new();
     fn recurse(
         circ: &Circuit,
         depth: usize,
         result: &mut String,
         seen_hashes: &mut HashMap<HashBytes, String>,
+        bijection: bool,
+        shape_only_necessary: bool,
     ) {
         result.push_str(&" ".repeat(depth * 2));
         if let Some(prev) = seen_hashes.get(&circ.info().hash) {
@@ -25,19 +31,36 @@ pub fn repr_circuit_deep_compiler(circuit: &Circuit) -> String {
             result.push('\n');
             return;
         }
+        let variant_string = circ.variant_string();
+        let variant_string_simplified = variant_string
+            .strip_suffix("Constant")
+            .unwrap_or(&variant_string);
         seen_hashes.insert(
             circ.info().hash,
-            seen_hashes.len().to_string() + " " + circ.name().unwrap_or(&circ.variant_string()),
+            seen_hashes.len().to_string() + " " + circ.name().unwrap_or(variant_string_simplified),
         );
         result.push_str(&(seen_hashes.len() - 1).to_string());
         if let Some(n) = circ.name() {
             result.push(' ');
-            result.push_str(n);
+            if bijection {
+                result.push('\'');
+                result.push_str(&n.replace(r"\", r"\\").replace("'", r"\'"));
+                result.push('\'');
+            } else {
+                result.push_str(n);
+            }
+        }
+        if !shape_only_necessary
+            || matches!(
+                circ,
+                Circuit::ScalarConstant(_) | Circuit::Scatter(_) | Circuit::Symbol(_)
+            )
+        {
+            result.push_str(&format!(" {:?}", circ.info().shape));
         }
         result.push(' ');
-        result.push_str(&format!("{:?}", circ.info().shape));
-        result.push(' ');
-        if circ.info().numel() > BigUint::from(400_000_000usize)
+        if !bijection
+            && circ.info().numel() > BigUint::from(400_000_000usize)
             && !matches!(circ, Circuit::ArrayConstant(_))
         {
             result.push_str(&format!(
@@ -45,15 +68,29 @@ pub fn repr_circuit_deep_compiler(circuit: &Circuit) -> String {
                 oom_fmt(circ.info().numel())
             ));
         }
-        result.push_str(&circ.variant_string());
+        result.push_str(variant_string_simplified);
         result.push(' ');
         result.push_str(&{
             match circ {
-                Circuit::ScalarConstant(scalar) => format!("{:10.5e}", scalar.value),
-                Circuit::Rearrange(rearrange) => rearrange.spec.to_einops_string(),
+                Circuit::ScalarConstant(scalar) => {
+                    format!("{:.}", scalar.value)
+                }
+                Circuit::Rearrange(rearrange) => rearrange.spec.to_einops_string(true),
                 Circuit::Einsum(einsum) => einsum.get_spec().to_einsum_string(),
-                Circuit::Index(index) => format!("{}", index.index),
-                Circuit::Scatter(scatter) => format!("{}", scatter.index),
+                Circuit::Index(index) => {
+                    if bijection {
+                        index.index.repr_bijection()
+                    } else {
+                        format!("{}", index.index)
+                    }
+                }
+                Circuit::Scatter(scatter) => {
+                    if bijection {
+                        scatter.index.repr_bijection()
+                    } else {
+                        format!("{}", scatter.index)
+                    }
+                }
                 Circuit::Concat(concat) => concat.axis.to_string(),
                 Circuit::GeneralFunction(gf) => gf.spec.name.clone(),
                 Circuit::Symbol(sy) => {
@@ -66,10 +103,18 @@ pub fn repr_circuit_deep_compiler(circuit: &Circuit) -> String {
                 Circuit::ModuleNode(mn) => {
                     format!("{}", &mn.spec.name.as_ref().unwrap_or(&"".to_owned()))
                 }
+                Circuit::ArrayConstant(ac) => {
+                    if bijection {
+                        ac.save_rrfs().unwrap();
+                        ac.tensor_hash_base16()[..14].to_owned()
+                    } else {
+                        "".to_owned()
+                    }
+                }
                 _ => "".to_owned(),
             }
         });
-        if let Some(real_named_axes)= &circ.info().named_axes && real_named_axes.iter().any(|x| x.is_some()) {
+        if let Some(real_named_axes)= &circ.info().named_axes && real_named_axes.iter().any(|x| x.is_some()) && !bijection {
             result.push_str(&format!(
                 " NA[{}]",
                 circ.info()
@@ -85,11 +130,25 @@ pub fn repr_circuit_deep_compiler(circuit: &Circuit) -> String {
         }
         result.push('\n');
         for child in circ.children() {
-            recurse(&child, depth + 1, result, seen_hashes);
+            recurse(
+                &child,
+                depth + 1,
+                result,
+                seen_hashes,
+                bijection,
+                shape_only_necessary,
+            );
         }
     }
     let mut result = String::new();
-    recurse(circuit, 0, &mut result, &mut seen_hashes);
+    recurse(
+        circuit,
+        0,
+        &mut result,
+        &mut seen_hashes,
+        bijection,
+        shape_only_necessesary,
+    );
     result
 }
 
